unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, IdBaseComponent, IdComponent, IdTCPServer;

type
  TfmServer = class(TForm)
    sbAttiva: TSpeedButton;
    Label1: TLabel;
    lbConnClients: TLabel;
    TCPServer: TIdTCPServer;
    procedure sbAttivaClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TCPServerConnect(AThread: TIdPeerThread);
    procedure TCPServerDisconnect(AThread: TIdPeerThread);
    procedure TCPServerExecute(AThread: TIdPeerThread);
  private
    { Private declarations }
    procedure IncClientCount;
    procedure DecClientCount;
  public
    { Public declarations }
    procedure Broadcast(msg: String);
    procedure ShutDown; 
  end;

var
  fmServer: TfmServer;

implementation

{$R *.DFM}

procedure TfmServer.sbAttivaClick(Sender: TObject);
begin
  // se il pulsante  gi, attiviamo il server
  if sbAttiva.Down then TCPServer.Active:=true
  else
  // altrimenti lo disattiviamo
    ShutDown;
end;

procedure TfmServer.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // alla chiusura della form disattiviamo il server
  ShutDown;
end;

procedure TfmServer.DecClientCount;
begin
  lbConnClients.Caption:=IntToStr(StrToIntDef(lbConnClients.Caption,1)-1);
  {
   qui facciamo questo: prendiamo il testo della label lbConnClients e
   lo trasformiamo in un integer, se non ha un valore valido prendiamo
   come valore 1, poi a questo numero sottraiamo uno, il risultato lo
   trasformiamo in string e lo assegnamo alla caption della label stessa
  }
end;

procedure TfmServer.IncClientCount;
begin
  lbConnClients.Caption:=IntToStr(StrToIntDef(lbConnClients.Caption,0)+1);
  {
   come sopra, ma adesso addizioniamo 1
  }
end;

procedure TfmServer.Broadcast(msg: String);
var x: integer;
    ClientList: TList;
begin
  {
   Prima di tutto dobbiamo bloccare i thread per evitare casini vari. Per farlo
   utilizziamo la procedura LockList, che restituisce una TList, che
   memorizziamo in ClientList per accedervi
  }
  ClientList:=TCPServer.Threads.LockList;
  try
    // ClientList.Count  un integer che indica il numero di elementi contenuti
    for x:=0 to ClientList.Count-1 do
      {
       per x da 0 al numero dei thread... (sottraiamo 1 perch la propriet
       count  zero-based, cio il primo elemento  il numero 0, il secondo 
       il numero 1 e cos via)
      }
      TIdPeerThread(ClientList.Items[x]).Connection.WriteLn(msg);
      {
       prendiamo l'x-esimo elemento della lista, facciamo un typecast a
       TIdPeerThread, richiamiamo la propriet connection
       (TIdTCPServerConnection) e utilizziamo il metodo WriteLn, a cui passiamo
       come parametro il messaggio da inviare.
      }
  finally
    TCPServer.Threads.UnlockList;
    {
     qualunque cosa succeda qui sopra, anche in caso di errori, alla fine
     sblocca la lista
    }
  end;
end;

procedure TfmServer.TCPServerConnect(AThread: TIdPeerThread);
begin
  {
   Stiamo operando dall'interno di un thread. Siccome accederemo ai
   componenti della VCL, dobbiamo utilizzare il metodo Synchronize
  }
  AThread.Synchronize(IncClientCount);
end;

procedure TfmServer.TCPServerDisconnect(AThread: TIdPeerThread);
begin
  // come sopra
  AThread.Synchronize(DecClientCount);
end;

procedure TfmServer.TCPServerExecute(AThread: TIdPeerThread);
begin
  {
    Finch il client  connesso, leggiamo i dati che
    ci invia e li rispediamo a tutti gli altri i
    client
  }
  while AThread.Connection.Connected do
    Broadcast(AThread.Connection.ReadLn);
end;

procedure TfmServer.ShutDown;
var x: integer;
    ClientList: TList;
begin
  {
    abbastanza ovvio quello che fa questo codice.
   Se qualcosa non  chiara vedete i commenti
   della procedura Broadcast
  }
  ClientList:=TCPServer.Threads.LockList;
  try
    for x:=0 to ClientList.Count-1 do
      TIdPeerThread(ClientList.Items[x]).Connection.Disconnect;
  finally
    TCPServer.Threads.UnlockList;
    TCPServer.Active:=false;
  end;
end;

end.
